/*
Software for the nanobot pcb
(C) 2014 Jeroen Domburg (jeroen AT spritesmods.com)

This program is free software: you can redistribute it and/or modify
t under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <ftdi.h>
#include <gd.h>

struct ftdi_context ftdi;
unsigned char sendbuff[10240*1024];
int sendpos;

/*
This is the code for the original PCB, with 5 16-bit shift registers to drive 
all 80 coils. The 90mA the drivers could sink proved to be too wimpy to move
the magnet.
*/


/*
Pinout for the PCB:
1 - ground
2 - /latch
3 - clock
4 - data
5 - Vdd,logic
6 - Vdd,magn
*/

//The output->trace mapping isn't 1-to-1. The mappings for X and Y are defined here.
//Numbers are in hex because it counts easier, with the chips having 16 outputs.
static int xmapping[]={
	0xF,0xE,0xD,0xC,0xB,0xA,0x9,0x8,0x0,0x1,0x2,0x3,0x4,0x5,0x6,0x7,
	0x1F,0x1E,0x1D,0x1C,0x1B,0x1A,0x19,0x18,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,
	0x2F,0x2E,0x2D,0x2C,0x2B,0x2A,0x29,0x28
};

static int ymapping[]={
	0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,
	0x3F,0x3E,0x3D,0x3C,0x3B,0x3A,0x39,0x38,0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,
	0x4F,0x4E,0x4D,0x4C,0x4B,0x4A,0x49,0x48,0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47
};

#define NO_ROWS 40
#define NO_COLS 40


//Masks for the pin signals
#define NLAT 0x10 //DTR, pin 2
#define CLK 0x02 //RX, pin 5
#define SIN 0x01 //TX, pin 1
#define NC  0x08 //CTS, pin 11


//Error handling routine.
int error(const char *err) {
	printf("Error at %s: %s\n", err, ftdi_get_error_string(&ftdi));
	exit(0); //Dirty. Disable to continue after errors.
	return 0;
}


void magnetize(int x, int y) {
	int i, b;
	for (i=NO_ROWS+NO_COLS+1; i>=0; i--) {
		b=0;
		if (x>=0) {
			if (xmapping[x]==i) b|=SIN;
//			if (xmapping[x-1]==i) b|=SIN;
//			if (xmapping[x+1]==i) b|=SIN;
		}
		if (y>=0) {
			if (ymapping[y]==i) b|=SIN;
//			if (xmapping[x-1]==i) b|=SIN;
//			if (ymapping[y-1]==i) b|=SIN;
		}
		if (b) printf("%x: %x\n", i, b);
		sendbuff[sendpos++]=b;
		sendbuff[sendpos++]=b|CLK;
		sendbuff[sendpos++]=b|CLK;
		sendbuff[sendpos++]=b;
	}
	sendbuff[sendpos++]=NLAT;
	sendbuff[sendpos++]=NLAT|CLK;
	sendbuff[sendpos++]=0;
	if (ftdi_write_data(&ftdi, sendbuff, sendpos)<0) error("writing buffer");
	sendpos=0;
}

//Main function
int main(int argc, char **argv) {
	int x;
	//Open FTDI communications
	if (ftdi_init(&ftdi)<0) return error("init");
	if (ftdi_usb_open(&ftdi, 0x0403, 0x6001)<0) return error("open");
	if (ftdi_set_bitmode(&ftdi, SIN|CLK|NLAT, BITMODE_BITBANG)<0) error("bitmode");
	if (ftdi_set_baudrate(&ftdi, 3000000/4)<0) error("baudrate");
	ftdi_usb_purge_buffers(&ftdi);
	sendpos=0;

	while(1) {
		//Some testing stuff to try out the magnetize()-routine. As stated before, dur
		//to the hardware being too wimpy, this never worked.
		int seq[]={33,34,35,36,35,34,33,-1};
		for (x=0; seq[x]!=-1; x++) {
			magnetize(seq[x]-32,-1);
//			magnetize(0,40);
			usleep(100000);
			magnetize(-1,-1);
			usleep(600000);
		}
//		magnetize(21,21);
	}


}
